/***************************************************************************
  Goindol

  Functions to emulate the video hardware of the machine.

***************************************************************************/

#include "emu.h"
#include "includes/goindol.h"


/***************************************************************************

  Callbacks for the TileMap code

***************************************************************************/

TILE_GET_INFO_MEMBER(goindol_state::get_fg_tile_info)
{
	int code = m_fg_videoram[2 * tile_index + 1];
	int attr = m_fg_videoram[2 * tile_index];
	SET_TILE_INFO_MEMBER(
			0,
			code | ((attr & 0x7) << 8) | (m_char_bank << 11),
			(attr & 0xf8) >> 3,
			0);
}

TILE_GET_INFO_MEMBER(goindol_state::get_bg_tile_info)
{
	int code = m_bg_videoram[2 * tile_index + 1];
	int attr = m_bg_videoram[2 * tile_index];
	SET_TILE_INFO_MEMBER(
			1,
			code | ((attr & 0x7) << 8) | (m_char_bank << 11),
			(attr & 0xf8) >> 3,
			0);
}



/***************************************************************************

  Start the video hardware emulation.

***************************************************************************/

void goindol_state::video_start()
{
	m_bg_tilemap = &machine().tilemap().create(tilemap_get_info_delegate(FUNC(goindol_state::get_bg_tile_info),this), TILEMAP_SCAN_ROWS, 8, 8, 32, 32);
	m_fg_tilemap = &machine().tilemap().create(tilemap_get_info_delegate(FUNC(goindol_state::get_fg_tile_info),this), TILEMAP_SCAN_ROWS, 8, 8, 32, 32);

	m_fg_tilemap->set_transparent_pen(0);
}


/***************************************************************************

  Memory handlers

***************************************************************************/

WRITE8_MEMBER(goindol_state::goindol_fg_videoram_w)
{
	m_fg_videoram[offset] = data;
	m_fg_tilemap->mark_tile_dirty(offset / 2);
}

WRITE8_MEMBER(goindol_state::goindol_bg_videoram_w)
{
	m_bg_videoram[offset] = data;
	m_bg_tilemap->mark_tile_dirty(offset / 2);
}



/***************************************************************************

  Display refresh

***************************************************************************/

static void draw_sprites( running_machine &machine, bitmap_ind16 &bitmap, const rectangle &cliprect, int gfxbank, UINT8 *sprite_ram )
{
	goindol_state *state = machine.driver_data<goindol_state>();
	int offs, sx, sy, tile, palette;

	for (offs = 0; offs < state->m_spriteram.bytes(); offs += 4)
	{
		sx = sprite_ram[offs];
		sy = 240 - sprite_ram[offs + 1];

		if (state->flip_screen())
		{
			sx = 248 - sx;
			sy = 248 - sy;
		}

		if ((sprite_ram[offs + 1] >> 3) && (sx < 248))
		{
			tile = ((sprite_ram[offs + 3]) + ((sprite_ram[offs + 2] & 7) << 8));
			tile += tile;
			palette = sprite_ram[offs + 2] >> 3;

			drawgfx_transpen(bitmap,cliprect,
						machine.gfx[gfxbank],
						tile,
						palette,
						state->flip_screen(),state->flip_screen(),
						sx,sy, 0);
			drawgfx_transpen(bitmap,cliprect,
						machine.gfx[gfxbank],
						tile+1,
						palette,
						state->flip_screen(),state->flip_screen(),
						sx,sy + (state->flip_screen() ? -8 : 8), 0);
		}
	}
}

SCREEN_UPDATE_IND16( goindol )
{
	goindol_state *state = screen.machine().driver_data<goindol_state>();
	state->m_fg_tilemap->set_scrollx(0, *state->m_fg_scrollx);
	state->m_fg_tilemap->set_scrolly(0, *state->m_fg_scrolly);

	state->m_bg_tilemap->draw(bitmap, cliprect, 0, 0);
	state->m_fg_tilemap->draw(bitmap, cliprect, 0, 0);
	draw_sprites(screen.machine(), bitmap, cliprect, 1, state->m_spriteram);
	draw_sprites(screen.machine(), bitmap, cliprect, 0, state->m_spriteram2);
	return 0;
}
